@@ -6,11 +6,7 @@
 #ifndef BSWAP_H
 #define BSWAP_H
 
-#include "config.h"
-
-#ifdef HAVE_BYTESWAP_H
-#include <byteswap.h>
-#else
+#include "common.h"
 
 static inline uint16_t bswap_16(uint16_t x){
     return (x>>8) | (x<<8);
@@ -33,8 +29,6 @@ static inline uint64_t bswap_64(uint64_t x)
     return r.ll;
 }
 
-#endif  /* !HAVE_BYTESWAP_H */
-
 // be2me ... BigEndian to MachineEndian
 // le2me ... LittleEndian to MachineEndian
 
