@@ -1,6 +1,6 @@
 /**
  * Flake: FLAC audio encoder
- * Copyright (c) 2006  Justin Ruggles <jruggle@earthlink.net>
+ * Copyright (c) 2006  Justin Ruggles
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -22,18 +22,11 @@
  * WAV decoder
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <inttypes.h>
+#include "common.h"
 
 #include "wav.h"
 #include "bswap.h"
 
-#define MAX(a,b) ((a) > (b) ? (a) : (b))
-#define MIN(a,b) ((a) > (b) ? (b) : (a))
-#define CLIP(x,min,max) MAX(MIN((x), (max)), (min))
-
 #define RIFF_ID     0x46464952
 #define WAVE_ID     0x45564157
 #define FMT__ID     0x20746D66
@@ -47,7 +40,7 @@ read4le(FILE *fp)
     return le2me_32(x);
 }
 
-static uint16_t
+static inline uint16_t
 read2le(FILE *fp)
 {
     uint16_t x;
@@ -65,7 +58,7 @@ wavfile_init(WavFile *wf, FILE *fp)
     memset(wf, 0, sizeof(WavFile));
     wf->fp = fp;
 
-    /* attempt to get file size */
+    // attempt to get file size
     wf->file_size = 0;
     wf->seekable = !fseek(fp, 0, SEEK_END);
     if(wf->seekable) {
@@ -93,32 +86,24 @@ wavfile_init(WavFile *wf, FILE *fp)
             case FMT__ID:
                 if(chunksize < 16) return -1;
                 wf->format = read2le(fp);
-                wf->filepos += 2;
                 wf->channels = read2le(fp);
-                wf->filepos += 2;
-                if(wf->channels == 0) return -1;
                 wf->sample_rate = read4le(fp);
-                wf->filepos += 4;
-                if(wf->sample_rate == 0) return -1;
                 read4le(fp);
-                wf->filepos += 4;
                 wf->block_align = read2le(fp);
-                wf->filepos += 2;
-                if(wf->block_align == 0) return -1;
                 wf->bit_width = read2le(fp);
-                wf->filepos += 2;
-                if(wf->bit_width == 0) return -1;
+                wf->filepos += 16;
+                if(!wf->channels || !wf->sample_rate || !wf->block_align || !wf->bit_width) {
+                    return -1;
+                }
                 chunksize -= 16;
 
                 // WAVE_FORMAT_EXTENSIBLE data
                 wf->ch_mask = 0;
                 if(wf->format == WAVE_FORMAT_EXTENSIBLE && chunksize >= 10) {
                     read4le(fp);    // skip CbSize and ValidBitsPerSample
-                    wf->filepos += 4;
                     wf->ch_mask = read4le(fp);
-                    wf->filepos += 4;
                     wf->format = read2le(fp);
-                    wf->filepos += 2;
+                    wf->filepos += 10;
                     chunksize -= 10;
                 }
 
@@ -513,16 +498,28 @@ fmt_convert(enum WavSampleFormat dest_fmt, void *dest,
 int
 wavfile_read_samples(WavFile *wf, void *output, int num_samples)
 {
-    int nr, bytes_needed, i, j, bps, nsmp, v;
+    int nr, i, j, bps, nsmp, v;
+    int convert = 1;
+    int read_size;
     uint8_t *buffer;
 
     if(wf == NULL || wf->fp == NULL || output == NULL) return -1;
     if(wf->block_align <= 0) return -1;
+
+    read_size = wf->block_align * num_samples;
+    if((wf->filepos + read_size) >= (wf->data_start + wf->data_size)) {
+        read_size = (wf->data_start + wf->data_size) - wf->filepos;
+        num_samples = read_size / wf->block_align;
+    }
     if(num_samples < 0) return -1;
     if(num_samples == 0) return 0;
 
-    bytes_needed = wf->block_align * num_samples;
-    buffer = calloc(bytes_needed, 1);
+    convert = (wf->read_format != wf->source_format);
+    if(convert) {
+        buffer = calloc(read_size, 1);
+    } else {
+        buffer = output;
+    }
 
     nr = fread(buffer, wf->block_align, num_samples, wf->fp);
     wf->filepos += nr * wf->block_align;
@@ -531,17 +528,20 @@ wavfile_read_samples(WavFile *wf, void *output, int num_samples)
     bps = wf->block_align / wf->channels;
     if(bps == 1) {
         if(wf->source_format != WAV_SAMPLE_FMT_U8) return -1;
-        fmt_convert(wf->read_format, output, wf->source_format, buffer, nsmp);
+        if(convert) {
+            fmt_convert(wf->read_format, output, wf->source_format, buffer, nsmp);
+        }
     } else if(bps == 2) {
-        int16_t *input = (int16_t *)buffer;
-#ifdef WORDS_BIG_ENDIAN
+#ifdef WORDS_BIGENDIAN
         uint16_t *buf16 = (uint16_t *)buffer;
         for(i=0; i<nsmp; i++) {
             buf16[i] = bswap_16(buf16[i]);
         }
 #endif
         if(wf->source_format != WAV_SAMPLE_FMT_S16) return -1;
-        fmt_convert(wf->read_format, output, wf->source_format, input, nsmp);
+        if(convert) {
+            fmt_convert(wf->read_format, output, wf->source_format, (int16_t *)buffer, nsmp);
+        }
     } else if(bps == 3) {
         int32_t *input = calloc(nsmp, sizeof(int32_t));
         for(i=0,j=0; i<nsmp*bps; i+=bps,j++) {
@@ -560,44 +560,43 @@ wavfile_read_samples(WavFile *wf, void *output, int num_samples)
                 wf->source_format != WAV_SAMPLE_FMT_S24) {
             return -1;
         }
-        fmt_convert(wf->read_format, output, wf->source_format, input, nsmp);
+        if(convert) {
+            fmt_convert(wf->read_format, output, wf->source_format, input, nsmp);
+        }
         free(input);
     } else if(bps == 4) {
-        if(wf->format == WAVE_FORMAT_IEEEFLOAT) {
-            float *input = (float *)buffer;
-#ifdef WORDS_BIG_ENDIAN
-            uint32_t *buf32 = (uint32_t *)buffer;
-            for(i=0; i<nsmp; i++) {
-                buf32[i] = bswap_32(buf32[i]);
-            }
+#ifdef WORDS_BIGENDIAN
+        uint32_t *buf32 = (uint32_t *)buffer;
+        for(i=0; i<nsmp; i++) {
+            buf32[i] = bswap_32(buf32[i]);
+        }
 #endif
+        if(wf->format == WAVE_FORMAT_IEEEFLOAT) {
             if(wf->source_format != WAV_SAMPLE_FMT_FLT) return -1;
-            fmt_convert(wf->read_format, output, wf->source_format, input, nsmp);
-        } else {
-            int64_t v64;
-            int32_t *input = calloc(nsmp, sizeof(int32_t));
-            for(i=0,j=0; i<nsmp*bps; i+=bps,j++) {
-                v64 = buffer[i] + (buffer[i+1] << 8) + (buffer[i+2] << 16) +
-                      (((uint32_t)buffer[i+3]) << 24);
-                if(v64 >= (1LL<<31)) v64 -= (1LL<<32);
-                input[j] = v64;
+            if(convert) {
+                fmt_convert(wf->read_format, output, wf->source_format, (float *)buffer, nsmp);
             }
+        } else {
             if(wf->source_format != WAV_SAMPLE_FMT_S32) return -1;
-            fmt_convert(wf->read_format, output, wf->source_format, input, nsmp);
-            free(input);
+            if(convert) {
+                fmt_convert(wf->read_format, output, wf->source_format, (int32_t *)buffer, nsmp);
+            }
         }
     } else if(wf->format == WAVE_FORMAT_IEEEFLOAT && bps == 8) {
-        double *input = (double *)buffer;
-#ifdef WORDS_BIG_ENDIAN
+#ifdef WORDS_BIGENDIAN
         uint64_t *buf64 = (uint64_t *)buffer;
         for(i=0; i<nsmp; i++) {
             buf64[i] = bswap_64(buf64[i]);
         }
 #endif
         if(wf->source_format != WAV_SAMPLE_FMT_DBL) return -1;
-        fmt_convert(wf->read_format, output, wf->source_format, input, nsmp);
+        if(convert) {
+            fmt_convert(wf->read_format, output, wf->source_format, (double *)buffer, nsmp);
+        }
+    }
+    if(convert) {
+        free(buffer);
     }
-    free(buffer);
 
     return nr;
 }
