@@ -1,6 +1,6 @@
 /**
  * Flake: FLAC audio encoder
- * Copyright (c) 2006  Justin Ruggles <jruggle@earthlink.net>
+ * Copyright (c) 2006  Justin Ruggles
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,9 +20,7 @@
 #ifndef CRC_H
 #define CRC_H
 
-#include "config.h"
-
-#include <inttypes.h>
+#include "common.h"
 
 extern void crc_init(void);
 
