@@ -1,6 +1,6 @@
 /**
  * Flake: FLAC audio encoder
- * Copyright (c) 2006  Justin Ruggles <jruggle@earthlink.net>
+ * Copyright (c) 2006  Justin Ruggles
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,20 +20,10 @@
 #ifndef LPC_H
 #define LPC_H
 
-#include "config.h"
-
-#include <stdlib.h>
-#include <inttypes.h>
+#include "common.h"
 
 #define MAX_LPC_ORDER 32
 
-#define ORDER_METHOD_MAX    0
-#define ORDER_METHOD_EST    1
-#define ORDER_METHOD_2LEVEL 2
-#define ORDER_METHOD_4LEVEL 3
-#define ORDER_METHOD_8LEVEL 4
-#define ORDER_METHOD_SEARCH 5
-
 extern int lpc_calc_coefs(const int32_t *samples, int blocksize, int max_order,
                           int precision, int omethod,
                           int32_t coefs[][MAX_LPC_ORDER], int *shift);
